% This is the main code to calculate thermal profile, surfae heat flux, BDT
% depth etc.. 
% (1) At the beginning of the code, you need to assign Nusselt
% number (Nu_input) and erosion rate (E_dot_input). The Nu_input and
% E_dot_input can be row vectors, and the code loops through different combinations. 
% (2) You need deriv4thorder.m for the thermal solver. The code of
% deriv4thodrd is adopted from Schmeling, H., and Marquart, G., 2014. A scaling law for
% approximating porous hydrothermal convection by an equivalent thermal conductivity:
% theory and application to the cooling oceanic lithosphere. Geophysical Journal International, 197(2), 645-664.
% (3) You need YSE.m to calculate the yield stress envelope
% (4) You can plot and/or save output plots by assign related tokens to 1
% (5) The code is developed based on Matlab_R2016b
% Please contact Wenrong Cao (wenrongc@unr.edu) if you have any questions
% 2019-2-14

clear
close all
warning off

Nu_input    = [1];                        % Nusselt number, 1 = no hydrothermal circulation, Nu_input >1
E_dot_input = [0];                        % exhumation rate [km/Myr]

plotstep        =   500;                  % plot every how many steps
                                          % adjust to get desired plot resolution. Smaller plotstep = higher resolution, slower run 
duration        =   10;                   % [Myr] duration of the simulation
TracerDepth     =   [7.5e3;20e3;40e3];    % [m] tracker depth at time =0 
k_phi_crit      =   1e-17;                % critical premeability for hydrothermal ciculation [m^2]

%------------------------------------------------
%     Options of plot and/or save output plots
%------------------------------------------------
%      Plot                     Save
%               1= Yes, 0= No
%------------------------------------------------
PLOT_ALL      =  0;         SAVE_ALL          =  0;
 
plotTemp      =  1;         saveTemp          =  0;   %  temperature profile                                                  
plotdiff      =  0;         savediff          =  0;   %  plot temperature difference against pure conduction case

plotconductivity =0;        saveconductivity  =0;  %  conductivity profile
plotkphi        = 0;        savekphi          =0;  %  premeability profile

plotFlux        = 1;        saveFlux         = 0;  %  total surface heat flux

plotTracerT     = 0;        saveTracerT      = 0;  %  temperature of tracked rock parcels
plotCoolingRate = 0;        saveCoolingRate  = 0;  %  cooling rate of parcels, to plot cooling rate, plotTracerT should be switched on

plotBDT         = 0;        saveBDT          = 0;  %  BDT depth (350C isotherm)
plotyse         = 0;        saveyse          = 0;  %  Yield Stress Envelope (YSE)

if PLOT_ALL ==1 
plotTemp=1;plotnorm=1;plotdiff=1; plotconductivity=1;plotkphi=1;plotFlux=1;  
plotTracerT=1;plotCoolingRate=1;plotBDT=1;plotyse=1; 
end
if SAVE_ALL ==1 
saveTemp=1;savenorm=1;savediff=1;saveconductivity=1;savekphi=1;saveFlux=1;  
saveTracerT=1;saveCoolingRate=1;saveBDT=1;saveyse=1; 
end

%---------------------
% Physical parameters
%---------------------
numtracer      =   size(TracerDepth,1);    %  Number of tracrs
H              =   100e3;                  %  Lithopshere thicknes [m] 
Hplot          =   60e3;                   %  plotted thickness [m]
Tsurface       =   0+273;                  %  Surface temperatur [K]
Tmoho          =   900+273;                %  Moho temperature [K]
Tmantle        =   1300+273;               %  Bottom Mantle temperature [K]
Cp             =   1000;                   %  Specific heat capacity of rock [J/kg/K]
rho            =   2800;                   %  Crustal density [kg/m3]
k_normal       =   3;                      %  normal themral conductivity [W/m/K]
kappa_normal   =   k_normal/Cp/rho;        %  normal themral diffusivity
secondsinyear  =   365.25*3600*24;         %  seconds in a year

  
for Nu = Nu_input
    
 for E_dot = E_dot_input
     
%----------------
% Establish grid
%----------------
nz             =   501;                    %   Number of gridpoints in z-direction
dz             =   H/(nz-1);               %   Spacing of grid
z              =   0:dz:H;                 %   generate grid in meter [m]
%----------------------
% Initialize particles
%----------------------
factor           =   150;             % Particle density factor (how many particles within one grid) 
Particles.z      =   0:dz/factor:H;
Particles.z      =   Particles.z(:);  % transfer from row to column vector
Particles.z      =   Particles.z + [rand(size(Particles.z))-0.5]*(dz/factor);
Particles.T          =   zeros(size(Particles.z)); % initialize particle temperature[K]
Particles.fullsize   =   size(Particles.z,1);      % record full size of Particles.z
k = k_normal.*ones(1,nz);                          % initialize conductiviey array   
%----------------------------------------
% Set inital temperature profile at gird
%----------------------------------------
Tmiddle          =  400+273  ;     % Middle temperaure [K]
Zmiddle          =  10e3;          % Depth that temperature zigzags [m]
Zmoho            =  60e3;          % moho depth [m]
T                =  ones(1,nz);
T(1)             =  Tsurface;
T(2:end)         =  Tmantle;

%Set inital zigzag temperature profile
ind     =   find(z<=Zmiddle);
T(ind)  =   z(ind).*(Tmiddle-Tsurface)/Zmiddle +Tsurface;
ind     =   find(z>Zmiddle & z <=H);
T(ind)  =   (z(ind)-Zmiddle)*(Tmantle-Tmiddle)/(H-Zmiddle)+Tmiddle;
Tnorm   =   T; % for comparison
%---------------------------------------------
% interpolate the Particles.T within the grid
%---------------------------------------------
Particles.T          =  interp1 (z,T,Particles.z,'spline','extrap');
Particles.Tnorm      =  interp1 (z,T,Particles.z,'spline','extrap'); % for comparison
  
% Before the loop determine permeability structure
% as it is not changed in the code. For all Nu cases
% when Nu=1 (no hydro case, the conductivity is NOT a function of phi)
    k_phi0 = 5e-13; % surface permeability (Saar and Manga, 2004)
    k_phi =  1e-14.*((z/1000).^(-3.2));
    ind = find (z<=800);
    k_phi(ind) = k_phi0.*exp(-z(ind)./250);
    
 % calculate reference penetration depth based on assigned critical phi and defined
 % permeability structure
 if  k_phi_crit >= 2.04e-14 
     z_hyr =  -log(k_phi_crit/5e-13)*0.25*1000;  % [m] 
 else
     z_hyr =  (k_phi_crit/1e-14)^(1/-3.2)*1000;  % [m] 
 end

% Draw initial temperatur profile    
 if plotTemp==1            
    figure(1)
     plot(T-273,z/1e3,'r','LineWidth',1.5); 
     ylabel('Depth [km]','FontSize',20);
     xlabel('Temperature [^oC]','FontSize',20);
     title(['Thermal profile after ',num2str(0),' Myrs'],'FontSize',20);
     ylim([0 Hplot/1e3]);
     xlim([0 900]);
     hold on
     grid on
     axis ij
     drawnow 
 end 
 
%---------------
% Set loop time
%---------------
t = 0;                               % time in seconds
t_array = 0;
dt = 0;
t_end = duration*1e6*secondsinyear;  % define the end time
count = 0;                           % count many main loops
plotcount = 0;                       % count how many plots 
TIME = 0;
counterforline = 0;

%===========
% Time loop
%===========
while t<t_end
    TIME = round(t/secondsinyear/1e6,3); % time in Myr unit 
    
% update depth of tracers    
    TracerDepth = TracerDepth-dt*(E_dot*1e3/1e6/secondsinyear) ;  
      
% =====================================
% Calculate and adjust hydrothermal penetration depth
% ===================================== 

 if  Nu == 1    
     kappa =k_normal./Cp./rho;    % if there is no hydrotherm, calculate normal diffusivity   
     z_hy = 0;                    % hydrothermal penetration depth =0
 else
  
     %%%%%%%%%% first of all find BDT depth
     T_BDT = 350 + 273;         % BDT temperature [K]
     ind = max(find(T<=T_BDT)); % find BDT depth
     BDT_depth = z(ind); 
     %%%%%%%%%% compare BDT depth and reference penetration depth (z_hyr in paper)
    
         if BDT_depth >=z_hyr % if BDT is deeper than z_hy_0, use z_hy_0
         % Based on the Nu and z_hy, calculate the conductivity structure 
            k_surface = k_normal*Nu;   
            ind = find (z < z_hyr);
            k(ind) = k_surface*exp(log(1/Nu).*z(ind)/z_hyr);
            ind = find (z >= z_hyr);
            k(ind) = k_normal;     
            kappa =k./Cp./rho;  
            z_hy = z_hyr ; % real penetration depth is the reference penetration depth
         else
            % if BDT_depth < z_hy
            k_surface = k_normal*Nu;   
            ind = find (z < z_hyr);
            k(ind) = k_surface*exp(log(1/Nu).*z(ind)/z_hyr);   
            % below BDT_depth, use normal conductivity
            ind = find (z >= BDT_depth);
            k(ind) = k_normal;     
            kappa =k./Cp./rho;          
            z_hy = BDT_depth; % in this case, BDT depth is the z_hy
         end
        
 end 
    
%----------------
% Thermal solver
%----------------
% adjust time step by diffusion criterion
    kappamax = max(kappa);
    dtdiff   = 0.8*dz^2/kappamax;
    dt       = dtdiff;
    dt_Myr   = dt/secondsinyear/1e6;
    % optimize dt size to display time in a proper way
    % dt ranges from 2.4e-4 Myr to 2.6e-5 Myr for Nu ranges from 1 to 9
    if dt_Myr >= 2e-4
       dt_Myr = 2e-4;
    else 
       dt_Myr = 2e-5;
    end
    dt  = dt_Myr*secondsinyear*1e6;
% Finite difference algorithm to get T of next time step
    [Tfirst]=deriv4thorder(nz,dz,T,1);
    [Tsecond]=deriv4thorder(nz,dz,kappa.*Tfirst,1);     
    Tnew = T + dt*Tsecond;
    % fix boundary temperatures
    Tnew(1) = Tsurface;
    Tnew(nz) = Tmantle;
    T=Tnew;  % translate new temperature to T array
% Calculate pure conductive cooling evolution recorded in Tnorm 
 if plotdiff  == 1
    [Tfirst]=deriv4thorder(nz,dz,Tnorm,1);
    [Tsecond]=deriv4thorder(nz,dz,kappa_normal.*Tfirst,1);
    Tnewnorm = Tnorm + dt*Tsecond;  
    Tnewnorm(1) = Tsurface;
    Tnewnorm(nz) = Tmantle;
    Tnorm=Tnewnorm;
 end
    
if mod (count+1,plotstep)==0    
    
% plot temperature profile    
if plotTemp==1
     figure(1) 
       if TIME == 0.1||TIME ==0.2||TIME ==0.5||TIME ==1||TIME ==2||TIME ==5||TIME ==10 ||TIME ==20 
             plot(T-273,z/1e3,'LineWidth',2); 
             set(gca,'linewidth',1.5,'fontsize',12);
             lgd1= legend ('Initial','0.1 Myr','0.2 Myr','0.5 Myr','1 Myr','2 Myr','5 Myr','10 Myr');
             lgd1.LineWidth = 0.5;
             lgd1.Location = 'southwest';
             lgd1.FontSize = 15;
             lgd1.FontWeight = 'bold';
       end
     ylabel('Depth [km]','FontSize',20);
     xlabel('Temperature [^oC]','FontSize',20);
     title(['Thermal profile after ',num2str(TIME),' Myrs'],'FontSize',20);
     ylim([0 Hplot/1e3]);
     xlim([0 900]);
     hold on
     grid on
     axis ij
     drawnow  
end  
 
% plot temperature difference compared to pure conduction case
if plotdiff ==  1    
     figure(2)  
      if TIME == 0.1||TIME ==0.2||TIME ==0.5||TIME ==1||TIME ==2||TIME ==5||TIME ==10  
            plot(T-Tnorm,z/1e3,'LineWidth',2);   
            set(gca,'linewidth',1.5,'fontsize',20);
            lgd2= legend ('0.1 Myr','0.2 Myr','0.5 Myr','1 Myr','2 Myr','5 Myr','10 Myr');
            lgd2.LineWidth = 0.5;
            lgd2.Location = 'southeast';
            lgd2.FontSize = 15;
            lgd2.FontWeight = 'bold';
      end
     pbaspect([1 0.618 1]);
     ylabel('Depth [km]','FontSize',20);
     xlabel('Temperature difference [^oC]','FontSize',20);
     title(['Temperature difference after ',num2str(TIME),' Myrs'],'FontSize',20);
     ylim([0 Hplot/1e3]);
     yticks([0:10:Hplot/1e3]);
     if E_dot ==10
     xlim([-150 400]);
     else
     xlim([-150 400]);
     end
     xticks([-100:100:400]);
     hold on
     grid on
     axis ij
     drawnow
end    

% plot total surface heat flux
if plotFlux ==1
   figure(3)
   ind = max(find(z<500));        % use temperature difference in the uppermost 500 m crust
   delta_T = T(ind)-Tsurface;
   delta_d = z(ind)-0;
   % get the conductivity at the surface
   ind2 = max(find(z<250));       % thermal conductivity at middle of 500 m crust, or at 250 m depth 
   total_surface_flux_total= k(ind2)*(delta_T/delta_d );
     set(gca,'linewidth',1.5,'fontsize',12);
     plot(TIME,(total_surface_flux_total*1e3),'r.'); % total surface heat flux
     ylabel('[mW/m^2]','FontSize',15);
     ylim([0 1000]);
     xlabel('Time [Myr]','FontSize',15);
     xlim([0 duration]);
     title(['Total surface heat flux after ',num2str(TIME),' Myrs'],'FontSize',15);
     grid on
     hold on
     drawnow
end
  
% plot BDT depth using 350C isothermal
if plotBDT == 1
   figure(4)  % BDT depth
   % calculate BDT depth
   T_BDT = 350 + 273;  % BDT temperature [K]
   ind = max(find(T<=T_BDT)); % use upper most 2 km 
   BDT_depth = z(ind); 
     set(gca,'linewidth',1.5,'fontsize',12);
     plot(TIME,BDT_depth/1e3,'b.-');
     ylabel('Depth [km]','FontSize',15);
     ylim([0 25]);
     yticks([0:2.5:25]);
     xlabel('Time [Myr]','FontSize',15);
     xlim([0 duration]);
     title(['BDT depth after ',num2str(TIME),' Myrs'],'FontSize',15);
     grid on
     hold on
     drawnow
end
     
% plot temperature of rock parcels
if  plotTracerT == 1
     figure(5) % tracker points temperature
     for j=1:numtracer
     ind_tracer(j) = min (find (Particles.z >= TracerDepth(j)));  
     end
     plot(TIME,Particles.T(ind_tracer)-273,'r.');
     set(gca,'FontSize',15);
     ylabel('Temperature [^oC]','FontSize',20);
     ylim([0 800]);
     xlabel('Time [Myr]','FontSize',20);
     xlim([0 duration]);
     xticks([0:1:duration]);
     title(['Tracer Temperature after ',num2str(TIME),' Myrs'],'FontSize',15);
     grid on
     hold on
     drawnow
     % Save Trackers temperature for cooling rate plot
     OldTracerTemp(1:numtracer,plotcount+1) = Particles.T(ind_tracer)-273;
end
      
% plot parcel cooling rate
if plotCoolingRate == 1
     figure(6) 
      if plotcount>0
      deltaTracerTemp = OldTracerTemp(1:numtracer,plotcount+1)-OldTracerTemp(1:numtracer,plotcount);
      coolingrate = deltaTracerTemp./(plotstep*dt/secondsinyear/1e6); % [C/Myr]  
     plot(TIME,coolingrate,'b.');
     set(gca,'FontSize',15);
     ylabel('Cooling rate [^oC/Myr]','FontSize',20);
     ylim([-500 100]);
     xlabel('Time [Myr]','FontSize',20);
     xlim([0 (duration)]);
     xticks([0:1:duration]);
     title(['Cooling rate after ',num2str(TIME),' Myrs'],'FontSize',15);
     grid on
     hold all 
     drawnow
      end 
     plotcount = plotcount+1;
end

% plot YSE  
if plotyse ==1 
    if TIME == 0.1||TIME ==0.2||TIME ==0.5||TIME ==1||TIME ==2||TIME ==5||TIME ==10  
    figure(7)
    YSE(z,T,TIME);  % call YSE function 
    end
end

% plot conductivity
if plotconductivity ==1 
     figure(21)
     plot(k,z/1e3,'r');
     ylabel('Depth [km]','FontSize',20);
     xlabel('k(eq) [W/m/K]','FontSize',20);
     title(['conductivity '],'FontSize',20);
     ylim([0 Hplot/1e3]);
     hold on
     grid on
     axis ij
     drawnow
end 
 
% plot premeability
if plotkphi == 1
     figure(22)
     plot(log10(k_phi),z/1e3,'r');
     ylabel('Depth [km]','FontSize',20);
     xlabel('log_1_0 kphi [m^2]','FontSize',20);
     title(['Permeability '],'FontSize',20);
     ylim([0 Hplot/1e3]);
     xlim([-20 -12]);
     hold on
     grid on
     axis ij
     drawnow
end

end

%----------------------------------------------
% interpolate the Particles.T within the grid
%----------------------------------------------
Particles.T         =  interp1 (z,T,Particles.z,'spline','extrap');
if plotdiff ==  1   %  do the interpolation for pure conductive case if needed
Particles.Tnorm     =  interp1 (z,Tnorm,Particles.z,'spline','extrap');
end

%------------------
% advect particles
%------------------
if E_dot > 0
     Particles.z     =  Particles.z-dt*(E_dot*1e3/1e6/secondsinyear);
     ind = find(Particles.z<0);   
     Particles.z(ind) =[];  % removed partciles having negative depth (above ground)
     Particles.T(ind) =[]; 
     Particles.T(1)  =  Tsurface;  % surface particles have surface temp.
     %-----------------------------------------------
     % patch hole of particles at the bottom of grid
     %-----------------------------------------------
     missingpart  =  Particles.fullsize-size((Particles.z),1);
     if missingpart >0    % if we miss any, do the patch 
         Particles.zadd      =   Particles.z(end):dz/factor:H;
         Particles.zadd      =   Particles.zadd(:);  %transfer from row to column vector
         Particles.zadd      =   Particles.zadd + [rand(size(Particles.zadd))-0.5]*(dz/factor);
         Particles.zadd      =   Particles.zadd(2:end);
         Particles.Tadd      =   Tmantle.*ones(size(Particles.zadd));
         Particles.z = [Particles.z; Particles.zadd];
         Particles.T = [Particles.T; Particles.Tadd]; 
     end
     
    % If you want to plot the actual distribution of particles
    % uncomment the following lines
        %     figure(999)
        %     plot (Particles.T-273,Particles.z/1e3);
        %     ylim([0 H/1e3]);
        %     xlim([0 Tmantle]);
        %     title(['Particles after ',num2str(TIME),' Myrs'],'FontSize',15);
        %     ylabel('Depth [km]','FontSize',20);
        %     xlabel('Temperature [^oC]','FontSize',20);
        %     grid on
        %     axis ij
        %     drawnow

%---------------------------------------------------------------
% After advection, translate temperature from particles to grid
%---------------------------------------------------------------    
     T    =  interp1 (Particles.z,Particles.T,z,'spline','extrap');   
end  % end of advect particles loop

    count = count+1;
    t=t+dt;  
    
end  % end of time loop

%--------------
% save as figures
%--------------

savename = ['Nu',num2str(Nu),'E',num2str(E_dot)];  

if saveTemp == 1
    h1= get(figure(1));  
    saveas(1,['0_Temp_' savename], 'pdf');  % save thermal profile  
    saveas(1,['0_Temp_' savename], 'fig');  % save matlab fig for edit
    close(1)
end

if savediff == 1
    h2= get(figure(2));  
    saveas(2,['0_Diff_' savename], 'pdf');  % save normalized profile 
    saveas(2,['0_Diff_' savename], 'fig');  % save normalized profile 
    close(2)
end

if saveFlux == 1
    h3= get(figure(3));  
    saveas(3,['0_Flux_' savename], 'pdf');  % save surface heat flux   
    saveas(3,['0_Flux_' savename], 'fig');  % save matlab fig for edit
    close(3)
end

if saveBDT == 1
    h4= get(figure(4));  
    saveas(4,['0_BDT_' savename], 'pdf');  % save BDT depth   
    saveas(4,['0_BDT_' savename], 'fig');  % save BDT depth   
    close(4)
end

if saveTracerT == 1
    h5= get(figure(5));  
    saveas(5,['0_Tracer_' savename], 'pdf');  % save temp of tracers 
    saveas(5,['0_Tracer_' savename], 'fig');  % save temp of tracers 
    close(5) 
end

if saveCoolingRate == 1
    h6= get(figure(6));  
    saveas(6,['0_CoolingRate_' savename], 'pdf');  % save cooling rate of rock parcels
    saveas(6,['0_CoolingRate_' savename], 'fig');  % save cooling rate of rock parcels
    close(6)
end

if saveyse == 1
    h7= get(figure(7));  
    saveas(7,['0_YSE_' savename], 'pdf');  % save YSE 
    saveas(7,['0_YSE_' savename], 'fig');  % save YSE 
    close(7)
end

if saveconductivity == 1
    h21= get(figure(21));  
    saveas(21,['0_Conductivity_' savename], 'pdf');  % save conductivity 
    saveas(21,['0_Conductivity_' savename], 'fig');  % save conductivity 
    close(21)
end

if savekphi == 1
    h22= get(figure(22));  
    saveas(22,['0_Premeability_' savename], 'pdf');  % save premeability
    saveas(22,['0_Premeability_' savename], 'fig');  % save premeability 
    close(22)
end


 end   % end of E_dot loop

end    % end of Nu loop



