% This is the script to plot Yield Stress Envelope as a functuin of crustal
% temperature profile
% input: z-depth in meter, T-temperature in K, t-time in Myr
% output: Stress (MPa)-Depth (km) plot 

function YSE(z,T,TIME)

% Define depth arrary
depth = z;       % depth from 0 to 40 km  [m]
Temp  = T;       % linear temperature [K]
u     = 0.75;    % coefficient of friction based on Byerlee's law
rho   = 2.8e3;   % Density of crustal rock [kg/m^3]
gamma = 0;       % pore fluid factor : gamma = fluid pressure/p*g*depth
g     = 9.8;     % gravity [m/s^2]

%-----------------------------------------------------------
% calculating differential stress for brittle shear failure
%-----------------------------------------------------------

% first to calculate some parameters when transform from normal and shear stresses to 
% principle stresses according to Anderson's theory
R = ((1+u^2)^(1/2)-u)^(-2);
alpha_n = (R-1)/R;  % for normal fault
alpha_t = R-1;    % for thrust fault
alpha_s = 2*(R-1)/(R+1); % for strike-slip fault

delta_stress_n = alpha_n*rho*g*depth*(1-gamma); % normal fault regime [Pa]
delta_stress_t = alpha_t*rho*g*depth*(1-gamma); % thrust fault regime [Pa]
delta_stress_s = alpha_s*rho*g*depth*(1-gamma); % strike-slip fault regime [Pa]

%plot (delta_stress_n/1e6,depth/1e3,'r');    % plot in MPa and km space;
%plot (delta_stress_t/1e6,depth/1e3,'b');    % plot in MPa and km space;
plot (delta_stress_s/1e6,depth/1e3,'k','LineWidth',2');     % plot "strike-slip" regime in MPa and km space;
hold on

%---------------------------------------------------------
% calculating differential stress for ductile deformation
%---------------------------------------------------------

% Wet quartzite flow law (Ranalli, 1995)
strainrate = 1e-15 ;     % strainrate [1/s]
R = 8.31;                % gas constant
A = 3.2e-4;              % [MPa^-n s-1]
n = 2.3;                 % 
Q = 154e3;               % [J]
Astar = [1e6.*A.^(-1./n)].^(-n); % translate to [Pa^-n S-1]
delta_stress_d = (strainrate/Astar./exp(-Q./(R.*Temp))).^(1/n); %[Pa]

% plot flow stress
plot (delta_stress_d/1e6,depth/1e3,'LineWidth',2');
axis ij
set(gca,'linewidth',1.5,'fontsize',15);
ylabel('Depth [km]','FontSize',20);
xlabel('Differential Stress [MPa]','FontSize',20);
title(['YSE profile after ',num2str(TIME),' Myrs'],'FontSize',20);
hold on
grid on
xlim([0 450]);   % stress range [MPa]
ylim([0 25]);   % depth range [km]
drawnow

end









